using System;
using System.Threading.Tasks;
using Newtonsoft.Json;
using System.Net.Http;
using System.Net;
using System.Text;
using WikiDotNet;

namespace svr.rpc.server.wiki
{
    class CdBridgeServer
    {
        // Initialize dataBridges server
        public dBridges.dBridges dbridge;
        public dBridges.remoteprocedure.Crpcserver wikiServer;

        private string dbApplnKey = 'appkey';
        private string dbAuthURL = 'auth_url';
        private string dbApplnSecret = 'appsecret';

        // Define exposed functions.
        // RPC: the client sends a single request and receives a single response.
        public async void summary(object inparameter, object response) {
            dBridges.Utils.dBParams parameter = inparameter as dBridges.Utils.dBParams;
            dBridges.responseHandler.CResponseHandler responsehandler = response as dBridges.responseHandler.CResponseHandler;
            var watch = new System.Diagnostics.Stopwatch();
            string payload = parameter.inparama;

            string searchstring = payload;
            watch.Start();
            WikiSearchSettings searchSettings = new WikiSearchSettings
            { RequestId = "Request ID", ResultLimit = 5, ResultOffset = 2, Language = "en" };

            WikiSearchResponse responsewiki = WikiSearcher.Search(searchstring, searchSettings);
            string resultstring = "";
            Dictionary<string, object> resultjson = new Dictionary<string, object>();
            resultjson.Add("statuscode", 0);
            foreach (WikiSearchResult result in responsewiki.Query.SearchResults) {
                 resultstring = resultstring + result.Preview;
            }
            watch.Stop();
            resultjson.Add("data", resultstring);
            resultjson.Add("exectime", watch.ElapsedMilliseconds);
            string jsonstring = JsonConvert.SerializeObject(resultjson);
            await responsehandler.end(jsonstring);
        }

        // Define all exposed functions.
        public void wikifunction(object eventInfo) {
            try
            {
                Action<object, object> icurrent = this.summary;
                (eventInfo as dBridges.remoteprocedure.Crpcserver).regfn("summary", icurrent);
            }
            catch (Exception e)
            {
                Console.WriteLine(e);
            }
        }


        public async Task init_dataBridges()
        {
            // Initialize dataBridges client
            this.dbridge = new dBridges.dBridges();

            // Replace your authentication url below, which you have received from dataBridges management portal.
            this.dbridge.auth_url = this.auth_url;

            // Replace your application key below, which you have received from dataBridges management portal.
            this.dbridge.appkey = this.dbApplnKey;

            // Replace your appsecret below, which you have received from dataBridges management portal.
            this.dbridge.appsecret = this.dbApplnSecret;

            // Initialize RPC Server wiki.search.
            this.wikiServer = dbridge.rpc.init("wiki.search");

            Action<object> ifunctions = this.wikifunction;
            this.wikiServer.functions = ifunctions;

            //Bind to connected event, Once the dataBridges is connected successfully, Subscription and other activities can be performed.
            Action<object> connected_callback;
            this.dbridge.connectionstate.bind("connected", connected_callback = async (object eventinfo) => {
                Console.WriteLine("dbridge.connectionstate.bind: event->connected" + "sessionid-> " + this.dbridge.sessionid);
                // Register RPC Server with dBrdige network.
                await this.wikiServer.register();

                // Bind to server.registration.success, This will be triggered When summary is ready to serve.
                Action<object, object> rconnectsuccess;
                this.wikiServer.bind("dbridges:rpc.server.registration.success", rconnectsuccess = (object payload, object metadata) => {
                    Console.WriteLine("dbridges:rpc.server.success {0} {1} {2} {3}", payload, metadata, this.wikiServer.getServerName(), this.wikiServer.isOnline());
                });
                // Bind to server.online, This will be triggered When summary is ready to serve.
                Action<object, object> rconnectonline;
                this.wikiServer.bind("dbridges:rpc.server.online", rconnectonline = (object payload, object metadata) => {
                    Console.WriteLine("dbridges:rpc.server.online {0} {1} {2} {3}", payload, metadata, this.wikiServer.getServerName(), this.wikiServer.isOnline());
                });
                // Bind to server.registration.fail, This will be triggered When any registration issue is raised.
                Action<object, object> rconnectfail;
                this.wikiServer.bind("dbridges:rpc.server.registration.fail", rconnectfail = (object payload, object metadata) => {
                    Console.WriteLine("dbridges:rpc.server.fail {0} {1} {2} {3}", payload, metadata, this.wikiServer.getServerName(), this.wikiServer.isOnline());
                });
            });

            //Bind to disconnected event, to get intimation about dataBridges network disconnection.
            Action<object> disconnected_callback;
            this.dbridge.connectionstate.bind("disconnected", disconnected_callback = (object eventinfo) => {
                Console.WriteLine("dbridge.connectionstate.bind: event->disconnected");
            });
            // Connect to dataBridges. If any runtime error it will be caught in catch().
            try
            {
                await this.dbridge.connect();
                string ch = Console.ReadLine();
            }
            catch (Exception e)
            {
                Console.WriteLine(e);
            }
        }
    }

    class Program
    {
        static void Main(string[] args)
        {
            CdBridgeServer db = new CdBridgeServer();
            Task.Run(db.init_dataBridges).Wait();
        }
    }
}